@@ -1,4 +1,4 @@
-robustRegH<-function(formula,data,tune=1.345,m=TRUE,max.it=1000,tol=1e-10){
+robustRegH<-function(formula,data,tune=1.345,m=TRUE,max.it=1000,tol=1e-6,anova.table=FALSE){
 
 psiHuber<-function(r,c){
 middle<-abs(r)<=c
@@ -7,15 +7,6 @@ low<-r<(-c)
 h<-middle*r + high*c + low*-c
 return(h)}
 
-derivPsiHuber<-function(r,c){
-true<-abs(r)<=c
-false<-(r<c*-1 || r>c)
-dph<-true*1 +false*0
-return(dph)
-}
-
-r3<-function(x){return(round(x,3))}
-
 bi<-FALSE
 if(m==FALSE){bi<-TRUE}
 
@@ -23,31 +14,28 @@ modelFrame=model.frame(formula,data)
 X=model.matrix(formula,data)
 y=model.extract(modelFrame,"response")
 
-beta=lm(formula,data)$coefficients
+model=lm.fit(X,y)
+b=model$coefficients
 n<-length(y)
-p<-length(beta)
-
-b<-beta
+p<-length(b)
 if(bi){
  tune<-(tune*sqrt(2*p*n))/(n-2*p)
- H<-X%*%solve(a=t(X)%*%X)%*%t(X)
- hii<-diag(H)
+ hii<-lm.influence(model)$hat
  pi<-(1-hii)/sqrt(hii)}
 
 convergence<-FALSE
 for(i in 1:max.it){
  b_temp<-b
- r<-y-X%*%b
+ r<-y-fit_rcpp(X,b) #replaced r<-y-X%*%b
  s<-median(abs(r-median(r)))/.6745
 
- if(m){rstar<-(r/s)}
- if(bi){rstar<-r/(s*pi)}
+ if(m){rstar<-(r/s)}else{rstar<-r/(s*pi)}
  
  psiH<-psiHuber(rstar,tune)
- w<-psiH/rstar  
- W<-diag(w[1:length(w),])  
- C<-solve(a=t(X)%*%W%*%X)
- b<-C%*%t(X)%*%W%*%y 
+ w<-psiH/rstar
+ b<-lm.wfit(x=X,y=y,w=w[,1])$coefficients
+
+ 
  if(i>10){
   if(sum(abs(b-b_temp))<tol){
    cat("\nRobust Regression with Huber Function\n")
@@ -56,10 +44,21 @@ for(i in 1:max.it){
    break}}
 }
 
-if(convergence==FALSE){b<-NULL}
-if(convergence){
+if(convergence==FALSE){b<-NULL;w<-NULL}
+if(convergence && anova.table){
+
+derivPsiHuber<-function(r,c){
+true<-abs(r)<=c
+false<-(r<c*-1 || r>c)
+dph<-true*1 +false*0
+return(dph)
+}
+	
+r3<-function(x){return(round(x,3))}
+r2<-function(x){return(round(x,2))}
+
  ybarw<-sum(y*w)/sum(w)
- ytild<-X%*%b
+ ytild=fit_rcpp(X,b) #replaced ytild<-X%*%b
  ssreg<-sum(w*(ytild-ybarw)^2)
  sserr<-sum(w*(y-ytild)^2)
  sstot<-sum(w*(y-ybarw)^2)
@@ -70,20 +69,23 @@ if(convergence){
  mse<-sserr/dferr
  sbsq<-(s^2*(n^2/(n-length(b)))*sum(psiH^2))/sum(derivPsiHuber(rstar,tune))^2
  F<-msr/sbsq
- c<-diag(C)
+ W<-Diagonal(x=w[1:length(w),]) 
+ c<-diag(x=solve(a=t(X)%*%W%*%X))
  sec<-sqrt(sbsq*c)
  t<-b/sec
- cat("source","\t","SS","\t","\t","df","\t","MS","\t","\t","F","\n")
- cat("model","\t",r3(ssreg),"\t",dfr,"\t",r3(msr),"\t",r3(F),"\n")
- cat("error","\t",sserr,"\t",dferr,"\t",r3(mse),"\n")
- cat("tot","\t",r3(sstot),"\t",dftot,"\n")
- cat("rsquared ",r3(ssreg/sstot),"\n","\n")
- row.names(b)[1]<-"intercept"
- estimates<-r3(cbind(b,sec,t))
- colnames(estimates)<-c("estimate","se","t")
+ 
+ cat("source  ","\t","SS","\t","\t","df","\t","MS","\t","\t","F","\n")
+ cat("model   ","\t",r2(ssreg),"\t",dfr,"\t",r2(msr),"\t",r2(F),"\n")
+ cat("error   ","\t",r2(sserr),"\t",dferr,"\n")
+ cat("tot     ","\t",r2(sstot),"\t",dftot,"\n")
+ cat("rsquared","\t",r3(ssreg/sstot),"\n")
+ cat("mse     ","\t",mse,"\n")
+ estimates<-cbind(b,sec,r2(t))
+ #colnames(estimates)<-c("estimate","se","t")
  print(estimates)
  }
- return(b)
-  
+
+ object=list("coefficients"=b,"weights"=w)
+ return(invisible(object)) 
 }
 
